module Practical6 where

import Data.List
{-

Task 1.
A cryptarithm is a mathematical puzzle where the digits in a sum
are replaced by letters, subject to the following rules:

1. Different letters stand for different digit.
2. No number has 0 as its leading digit.

To solve the cryptarithm, you have to find the number
represented by each letter.
Solve the cryptarithm below. Use the List monad.

  AB
+ BC
----
 BCB 

-}

{-
As requested, all of these solutions are given with do, bind
and list comprehension :)
-}
solution1_do :: [(Integer, Integer, Integer)]
solution1_do = do
    a <- [1..9]
    b <- [1..9] \\ [a]
    c <- [0..9] \\ [a, b]
    if (10*a + b + 10*b + c == 100*b + 10*c + b)
        then return (a, b, c)
        else []

solution1_bind :: [(Integer, Integer, Integer)]
solution1_bind =
    [1..9] >>= \a -> 
    [1..9] \\ [a] >>= \b ->
    [0..9] \\ [a, b] >>= \c ->
    if (10*a + b + 10*b + c == 100*b + 10*c + b)
        then return (a, b, c)
        else []

solution1_comp :: [(Integer, Integer, Integer)]
solution1_comp = [(a, b, c) | a <- [1..9], 
                              b <- [1..9] \\ [a],
                              c <- [0..9] \\ [a, b],
                              10*a + b + 10*b + c == 100*b + 10*c + b]
{-
Task 2.
Each card in a deck has a rank between 1 (Ace) and 13 (King)
and a Suit (Club, Diamond, Heart, Spade).
Use the list monad to generate all cards in a deck.
-}

data Suit = Club | Diamond | Heart | Spade deriving (Show, Eq)

solution2_do :: [(Integer, Suit)]
solution2_do = do
    x <- [1..13]
    s <- [Club, Diamond, Heart, Spade]
    return (x, s)

solution2_bind :: [(Integer, Suit)]
solution2_bind =
    [1..13] >>= \x ->
    [Club, Diamond, Heart, Spade] >>= \s ->
    return (x, s)

solution2_comp :: [(Integer, Suit)]
solution2_comp = [(x, s) | x <- [1..13], s <- [Club, Diamond, Heart, Spade]]
{-
Task 3.
Suppose you're on a game show, and you're given the choice between
two closed doors. Each door has a sign on it.
The sign on the left door says:
  "Behind this door is the goat, behind the other is the car."
The sign on the right door says:
  "Behind one of these doors is the goat, behind the other is the car."

The host of the game show tells you that exactly one of the signs is
correct. You know that the host is telling the truth.

You can open one door. You'll get whatever is behind it as
your prize.

Which door should you open to get the car?
Use the list monad to find out!
-}

data Room = Goat | Car deriving (Show, Eq)

-- when is the left sign true
signL :: Room -> Room -> Bool
signL l r = (l == Goat) && (r == Car)

signR :: Room -> Room -> Bool
signR l r = or [(l == Goat) && (r == Car), (l == Car) && (r == Goat)]

solution3_bind :: [(Room, Room)]
solution3_bind =
    [Goat, Car] >>= \l -> 
    [Goat, Car] >>= \r ->
    if or [signL l r && not (signR l r), not (signL l r) && signR l r]
        then return (l, r)
        else []

solution3_do :: [(Room, Room)]
solution3_do = do
    l <- [Goat, Car] 
    r <- [Goat, Car]
    if or [signL l r && not (signR l r), not (signL l r) && signR l r]
        then return (l, r)
        else []

solution3_comp :: [(Room, Room)]
solution3_comp = [(l, r) | l <- [Goat, Car], r <- [Goat, Car],
                           or [signL l r && not (signR l r), not (signL l r) && signR l r]]

solutionBonus_do :: [(Room, Room)]
solutionBonus_do = do
    l <- [Goat, Car]
    r <- [Goat, Car]
    if or [signL l r && not (signR l r), not (signL l r) && signR l r, not (signL l r) && not (signR l r)]
        then return (l, r)
        else []

solutionBonus_bind :: [(Room, Room)]
solutionBonus_bind =
    [Goat, Car] >>= \l ->
    [Goat, Car] >>= \r ->
    if or [signL l r && not (signR l r), not (signL l r) && signR l r, not (signL l r) && not (signR l r)]
        then return (l, r)
        else []

solutionBonus_comp :: [(Room, Room)]
solutionBonus_comp = [(l, r) | l <- [Goat, Car], r <- [Goat, Car],
                           or [signL l r && not (signR l r),
                               not (signL l r) && signR l r,
                               not (signL l r) && not (signR l r)]]

data Direction = L | R deriving (Show, Eq)

-- This was ommitted from the lecture, but you can use it to verify the probabilities for
-- yourself!

getProbability :: Room -> Direction -> Double
getProbability r d =  rs_in_door_d / total_length where
    rs_in_door_d = (fromIntegral $ length (filter (\x -> x == r) (map op solutionBonus_bind)))
    total_length = (fromIntegral $ length solutionBonus_bind)
    
    op = getOp d
    
    getOp L = fst
    getOp R = snd



{-
Bonus Task.
Suppose you're on a game show, and you're given the choice between
two closed doors. Each door has a sign on it.
The sign on the left door says:
  "Behind this door is a goat, behind the other is a car."
The sign on the right door says:
  "Behind one of these doors is a goat, behind the other is a car."

The host of the game show tells you that the prize behind each door
was chosen independently, by flipping a coin: head -> Goat, tail -> Car.
The host also tells you that at most one of the signs is
correct. You know that the host is telling the truth.

Which door should you open to maximize the probability of
winning the car? Use the list monad to find out.
-}
